
#include "S.h"

typedef struct s_object {
        int Type;
        char *name;
        int Mode;
        long Length;
        pointer value;
        union {
                struct s_object *frame;
                struct s_object *next;  /* for lists */
        } x;
        long nalloc;
        long status; 
} SO;

/* macro PMEL: */
/*   obtain j,k element of ith matrix of structure x, put in y */

#define PMEL(x,i,j,k,y) { \
long nr; \
pointer p; \
p = (*(x+((i)-1)))->value; \
nr = *(*(p.tree+1))->value.Long; \
y = *((*p.tree)->value.Double+ (k-1)*nr + j-1);  }

/* macro PMELPUT: */
/*   put to j,k element of ith matrix of structure x, put in y */

#define PMELPUT(x,i,j,k,y) { \
long nr; \
pointer p; \
p = (*(x+((i)-1)))->value; \
nr = *(*(p.tree+1))->value.Long; \
*((*p.tree)->value.Double+ (k-1)*nr + j-1) = y; }

/* macro GETDIM1: */
/* obtain dimensions of first element of list, nr nc must be ints */

#define GETDIM1(x,nr,nc) { \
pointer p; \
p = (*(x))->value; \
nr = *(*(p.tree+1))->value.Long; \
nc = *((*(p.tree+1))->value.Long+1); }

/* macro GETDIMi: */
/* obtain dimensions of ith element of list, nr nc must be ints */

#define GETDIMi(x,i,nr,nc) { \
pointer p; \
p = (*(x+((i)-1)))->value; \
nr = *(*(p.tree+1))->value.Long; \
nc = *((*(p.tree+1))->value.Long+1); }

/* macro SELLOC: */
/* location of data in an S matrix (row major) */
/* can be used for extraction or assignment */
/* need to know nrows of intended result */

#define SELLOC(dptr,nr,nc,j,k) \
*(dptr + (k-1)*nr + (j-1) )

void pmat_sum( SO** x, int* l, double* ans )
{
long nr, nc;
long i, j, k;
double y;

GETDIM1( x, nr, nc )

for ( i = 1 ; i <= *l ; i++ )
   {
   for (j = 1 ; j <= nr ; j++ )
      {
      for (k = 1 ; k <= nc; k++ )
	 {
	 PMEL(x,i,j,k,y)
	 SELLOC( ans, nr, nc, j, k ) += y;
	 }
      }
   }
}

void pmat_add( SO** x, SO** y, SO** z, int* l )
{
double a, b, c;
int i, j, k, nr, nc;

for ( i = 1 ; i <= *l ; i++ )
   {
   GETDIMi(x,i,nr,nc)
   for (j = 1 ; j <= nr ; j++ )
      {
      for (k = 1 ; k <= nc; k++ )
	 {
	 PMEL(x,i,j,k,a)
	 PMEL(y,i,j,k,b)
	 PMELPUT(z,i,j,k,a+b)
	 }
      }
    }
}

void pmat_mult( SO** x, SO** y, SO** z, int* l )
{
double a, b, c;
int i, j, k, m, nrx, ncx, nry, ncy;

for ( i = 1 ; i <= *l ; i++ )
   {
   GETDIMi(x,i,nrx,ncx)
   GETDIMi(y,i,nry,ncy)
   for (j = 1 ; j <= nrx ; j++ )
      {
      for (k = 1 ; k <= ncy; k++ )
	 {
         c = 0.;
         for (m = 1; m <= ncx; m++ )
	   {
	   PMEL(x,i,j,m,a)
	   PMEL(y,i,m,k,b)
	   c += a*b;
           }
         PMELPUT(z,i,j,k,c)
	 }
      }
    }
}

